<?php
/* Plugin Initialize function */
add_action("init", 'aone_msg_init', 1);
function aone_msg_init(){
	global $wpdb;

	$wpdb->query(
		"CREATE TABLE IF NOT EXISTS `aone_messages` (
			`id`                 INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
			`from`		         BIGINT(20) UNSIGNED,
			`with`			     BIGINT(20) UNSIGNED,
			`type`		         VARCHAR(128) DEFAULT '',
			`targetid`		     BIGINT(20) UNSIGNED,
			`msgbody`		     TEXT,
			`unreadmsg`		     TEXT,
			`timestamp`		     DATETIME NOT NULL
		) ENGINE = INNODB
		DEFAULT CHARACTER SET = utf8
		COLLATE = utf8_general_ci"
	);
	
	$wpdb->query(
		"CREATE TABLE IF NOT EXISTS `aone_tempchat` (
			`id`                 INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
			`from`		         BIGINT(20) UNSIGNED,
			`to`			     BIGINT(20) UNSIGNED,
			`type`		         VARCHAR(128) DEFAULT '',
			`targetid`		     BIGINT(20) UNSIGNED,
			`message`		     TEXT,
			`mode`            	 VARCHAR(128) DEFAULT '',
			`timestamp`		     DATETIME NOT NULL
		) ENGINE = INNODB
		DEFAULT CHARACTER SET = utf8
		COLLATE = utf8_general_ci"
	);
	
	aone_msg_add_column_if_not_exist('aone_messages', 'type', $column_attr = "VARCHAR(128) DEFAULT ''" );
	aone_msg_add_column_if_not_exist('aone_messages', 'targetid', $column_attr = "BIGINT(20) UNSIGNED" );
	
	aone_msg_add_column_if_not_exist('aone_tempchat', 'type', $column_attr = "VARCHAR(128) DEFAULT ''" );
	aone_msg_add_column_if_not_exist('aone_tempchat', 'targetid', $column_attr = "BIGINT(20) UNSIGNED" );
	
}

/* Alter table for new fields */
function aone_msg_add_column_if_not_exist($db, $column, $column_attr = "VARCHAR( 255 ) NULL" ){
	global $wpdb;
	$exists = false;
	$results = $wpdb->get_results("show columns from ".$db);
	foreach($results as $result){
		if($result->Field == $column){
			$exists = true;
			break;
		}
	}
	if(!$exists){
		$wpdb->query("ALTER TABLE `$db` ADD `$column`  $column_attr");
	}
}

/* Enqueue Scripts */
add_action('wp_enqueue_scripts', 'aone_msg_enqueue_scripts', 99);
function aone_msg_enqueue_scripts(){
	global $service_finder_options,$current_user;
	
	/* Enqueue Style */
	wp_register_style('font-awesome', AONE_MSG_URI . 'css/font-awesome.min.css','',null);
	wp_enqueue_style('font-awesome');
	
	wp_register_style('aone_msg_style', AONE_MSG_URI . 'css/chatbox.css','',null);
	wp_enqueue_style('aone_msg_style');
	
	if(is_rtl()){
	wp_register_style('aone_msg_style_rtl', AONE_MSG_URI . 'css/chatbox-rtl.css','',null);
	wp_enqueue_style('aone_msg_style_rtl');
	}
	
	wp_register_style('uploadifive', AONE_MSG_URI . 'lib/uploadifive/uploadifive.css','',null);
	wp_enqueue_style('uploadifive');
	
	wp_register_style('jquery-emoji', AONE_MSG_URI . 'css/jquery.emoji.css','',null);
	wp_enqueue_style('jquery-emoji');
	
	wp_register_style('railscasts', AONE_MSG_URI . 'css/railscasts.css','',null);
	wp_enqueue_style('railscasts');
	
	
	/* Enqueue javascript */
	wp_register_script('aone_msg_js_mCustomScrollbar', AONE_MSG_URI . 'js/jquery.mCustomScrollbar.min.js','',null);
	wp_enqueue_script('aone_msg_js_mCustomScrollbar');
	
	
	
	if(is_user_logged_in())
	{
	if(aone_msg_get_user_role($current_user->ID) == 'Provider' || aone_msg_get_user_role($current_user->ID) == 'Customer' || aone_msg_get_user_role($current_user->ID) == 'administrator'){
	$userCap = array();
	if(class_exists('service_finder_booking_plugin')){
	$userCap = service_finder_get_capability($current_user->ID);
	}
	
	if((in_array('message-system',$userCap) && aone_msg_get_user_role($current_user->ID) == 'Provider') || aone_msg_get_user_role($current_user->ID) == 'Customer' || aone_msg_get_user_role($current_user->ID) == 'administrator'){
	wp_register_script('aone_msg_js_custom', AONE_MSG_URI . 'js/custom.js','',null);
	wp_enqueue_script('aone_msg_js_custom');
	}
	}
	}
	
	wp_register_script('uploadifive', AONE_MSG_URI . 'lib/uploadifive/jquery.uploadifive.min.js','',null);
	wp_enqueue_script('uploadifive');
	
	wp_register_script('jquery-emoji', AONE_MSG_URI . 'js/jquery.emoji.min.js','',null);
	wp_enqueue_script('jquery-emoji');
	
	$timeinterval = (!empty($service_finder_options['message-refresh-rate'])) ? $service_finder_options['message-refresh-rate'] : 30;
	
	$timeinterval = intval($timeinterval) * 1000;
	
	if(function_exists('service_finder_get_avatar_by_userid'))
	{
		$useravatar = service_finder_get_avatar_by_userid( $current_user->ID, 'thumbnail');
	}else
	{
		$useravatar = '';
	}
	
	wp_add_inline_script( 'aone_msg_js_custom', '/*Declare global variable*/
	var timeinterval = "'.$timeinterval.'";
	var uploadfile = "'.AONE_MSG_URI.'lib/uploadifive/uploadfile.php'.'";
	var currentuseravatar = "'.$useravatar.'";
	var currentusername = "'.aone_msg_user_data( 'display_name',$current_user->ID).'";
	', 'after' );
	require AONE_MSG_DIR. '/localize/localize.php'; //Localize
}

/* Add Inline script */
add_action('wp_head','aone_msg_ajax_url');
function aone_msg_ajax_url() { ?>
	<script type="text/javascript">
	// <![CDATA[
	var aone_ajax_url = '<?php echo admin_url('admin-ajax.php'); ?>';
	var aone_site_url = '<?php echo home_url(); ?>';
	var imgpath = '<?php echo AONE_MSG_URI.'img/'; ?>';
	var aoneparam;
	// ]]>
	</script>
<?php
}

/* Update users online status */
add_action('wp', 'aone_msg_update_online_users_status');
function aone_msg_update_online_users_status() {
    if (is_user_logged_in()) {
        if (($logged_in_users = get_transient('users_online')) === false) $logged_in_users = array();
        $current_user = wp_get_current_user();
        $current_user = $current_user->ID;
        $current_time = time();
        if (!isset($logged_in_users[$current_user]) || ($logged_in_users[$current_user] < ($current_time - (5 * 60)))) {
            $logged_in_users[$current_user] = $current_time;
            set_transient('users_online', $logged_in_users, 30 * 60);
        }
    }
}

/* Check user is online */
function aone_msg_is_user_online($user_id) {
    $logged_in_users = get_transient('users_online');
    return isset($logged_in_users[$user_id]) && ($logged_in_users[$user_id] > (time() - (15 * 60)));
}

/* Set user offline after logout */
function aone_msg_set_user_logged_out( $redirect_to, $requested_redirect_to, $user ) {
    
  	$logged_in_users = get_transient('users_online');
    if(!empty($logged_in_users) && (is_array($logged_in_users) || is_object($logged_in_users)))
	{
	if(isset($logged_in_users[$user->ID]))
	{
		unset($logged_in_users[$user->ID]);
	}
	}
	
	set_transient('users_online', $logged_in_users);
    
	if($requested_redirect_to != '')
	{
		return $requested_redirect_to;
	}else
	{
		return home_url();
	}
	
}
add_filter( 'logout_redirect', 'aone_msg_set_user_logged_out', 10, 3 );

/* Set user status login after user login */
add_action('wp_login', 'aone_msg_set_user_logged_in');
function aone_msg_set_user_logged_in() {
    if (($logged_in_users = get_transient('users_online')) === false) $logged_in_users = array();
    $current_user = wp_get_current_user();
    $current_user = $current_user->ID;
    $current_time = time();
    if (!isset($logged_in_users[$current_user]) || ($logged_in_users[$current_user] < ($current_time - (5 * 60)))) {
        $logged_in_users[$current_user] = $current_time;
        set_transient('users_online', $logged_in_users, 30 * 60);
    }
}

/* Display user status on site */
function aone_msg_display_user_status($user_id){
	 if (aone_msg_is_user_online($user_id)) {
		return '<span class="aon-user-online onlinestatus" data-userid="'.esc_attr($user_id).'"></span>';
	 } else {
		return '<span class="aon-user-offline onlinestatus" data-userid="'.esc_attr($user_id).'"></span>';
	 }
}

/* Display user status with text on site */
function aone_msg_display_user_status_with_text($user_id){
	 if (aone_msg_is_user_online($user_id)) {
		return '<span class="aon-messenger-availability-sign onlinestatus" data-userid="'.esc_attr($user_id).'">'.esc_html__('I am available','aone-messaging').'</span>';
	 } else {
		return '<span class="aon-user-unavailability-sign onlinestatus" data-userid="'.esc_attr($user_id).'">'.esc_html__('Not available','aone-messaging').'</span>';
	 }
}

add_action( 'wp_head', 'aone_msg_localization');
/*Include localize file*/
function aone_msg_localization() {
//require AONE_MSG_DIR. '/localize/localize.php'; //Localize
}

add_action( 'init', 'aone_msg_download_attachemnts' );
function aone_msg_download_attachemnts(){
	global $current_user;
	if ( isset( $_GET['secure_download'] ) && is_numeric( $_GET['secure_download'] ) ) {
		global $wpdb;
		global $current_user;
		$attachment_id = $_GET['secure_download'];

		$direct_download = 1;

		if ( user_can( $current_user, 'manage_options' ) || $direct_download ) {
			$file = get_attached_file( $attachment_id );
			if ( $file ) {
				$filename = basename( $file );

				$fp = @fopen( $file, 'rb' );

				if (strstr( $_SERVER['HTTP_USER_AGENT'], "MSIE") ) {
					header( 'Content-Type: "application/octet-stream"' );
					header( 'Content-Disposition: attachment; filename="' . $filename . '"' );
					header( 'Expires: 0' );
					header( 'Cache-Control: must-revalidate, post-check=0, pre-check=0' );
					header( 'Content-Transfer-Encoding: binary' );
					header( 'Pragma: public' );
					header( 'Content-Length: ' . filesize( $file ) );
				} else {
					header( 'Content-Type: "application/octet-stream"' );
					header( 'Content-Disposition: attachment; filename="' . $filename . '"' );
					header( 'Content-Transfer-Encoding: binary' );
					header( 'Expires: 0' );
					header( 'Pragma: no-cache' );
					header( 'Content-Length: ' . filesize( $file ) );
				}

				fpassthru( $fp );
				fclose( $fp );

				exit; // all good
			} else {
				exit( __( 'Wrong attachment ID!', 'aone-messaging' ) );
			}
		}
		exit( __( 'You are not allowed to download this!', 'aone-messaging' ) );

	}
}
?>
