<?php

if ( ! defined( 'ABSPATH' ) ) {

	exit; // Exit if accessed directly

}



require_once AONESMS_PLUGIN_DIR . '/Twilio/autoload.php';

use Twilio\Rest\Client;



/* Send twillio otp verification message */

add_action("wp_ajax_nopriv_otp_send_sms", "aonesms_fn_otp_send_sms");

add_action("wp_ajax_otp_send_sms", "aonesms_fn_otp_send_sms");

function aonesms_fn_otp_send_sms()

{

	global $wpdb,$aonesms_prefix,$service_finder_options;

	

	$countrycode = aonesms_get_data($_POST,'country_code');
	
	$phone = aonesms_get_data($_POST,'phone');

	

	$sid = aonesms_get_data($service_finder_options,'twillio-sid');

	$token = aonesms_get_data($service_finder_options,'twillio-token');

	$twiliosenderid = aonesms_get_data($service_finder_options,'twillio-senderid');

	

	$otpsize = 6;



	$otp = '';

	for ($i = 0; $i < $otpsize; $i++) {

		$otp .= rand(0, 9);

	}

	

	$message = "Your OTP for %SITENAME% is %OTP%";

    $message = str_replace("%SITENAME%", get_option('blogname'), $message);

    $message = str_replace("%OTP%", $otp, $message);

	

	$client = new Client($sid, $token);

	

	$tonumber = $countrycode.''.$phone;

	

	try {

		$client->messages->create(

			$tonumber,

			array(

				'From' => $twiliosenderid,

				'Body' => $message

			)

		);

		$countrycode = ltrim($countrycode,'+');
	
		$countrycode = intval($countrycode);

		$tablename = $aonesms_prefix.'mobile_otp';
		
		$row = $wpdb->get_row($wpdb->prepare('SELECT * FROM '.$tablename.' WHERE `countrycode` = %d AND `phone` = %s',$countrycode,$phone));
		
		if(!empty($row))
		{
		
		$data = array(

			'otp' => $otp,

			'time' => date("Y-m-d H:i:s",strtotime("now"))

		);
		
		$where = array(
			'countrycode' => $countrycode,

			'phone' => $phone,
			);
		
		$wpdb->update($tablename,wp_unslash($data),$where);
		
		}else{
		
		$data = array(

			'countrycode' => $countrycode,

			'phone' => $phone,

			'otp' => $otp,

			'time' => date("Y-m-d H:i:s",strtotime("now"))

		);
		
		$wpdb->insert($tablename,wp_unslash($data));
		
		}
		
		$response = array(

			'message'   => esc_html__('OTP sent successfully.','aonesms'),

		);

		wp_send_json_success($response);

		

	}catch(Exception $e){

	

		$response = array(

			'message'   => $e->getMessage()

		);

		wp_send_json_error($response);

	}

	

	exit;

}



/* twillio otp verification process */

add_action("wp_ajax_nopriv_ajax_otp_verification", "aonesms_fn_otp_verification");

add_action("wp_ajax_ajax_otp_verification", "aonesms_fn_otp_verification");

function aonesms_fn_otp_verification()

{



	$countrycode = aonesms_get_data($_POST,'country_code');

	$phone = aonesms_get_data($_POST,'phone');

	$otp = aonesms_get_data($_POST,'otp');

	

	$verify = aonesms_verify_otp($phone,$countrycode,$otp);

	

	if (!$verify) {

		$response = array(

			'message'   => esc_html__('OTP verification is failed.','aonesms')

		);

		wp_send_json_error($response);

	}else

	{

		wp_send_json_success();

	}



exit;

}

// Verify OTP

function aonesms_verify_otp($phone,$country_code,$signupotp)

{

	global $wpdb,$aonesms_prefix;

  

	$country_code = filter_var($country_code, FILTER_SANITIZE_NUMBER_INT);

	$phone = filter_var($phone, FILTER_SANITIZE_NUMBER_INT);

	$country_code = ltrim($country_code,'+');
	
	$country_code = intval($country_code);

	$tablename = $aonesms_prefix.'mobile_otp';

	$row = $wpdb->get_row($wpdb->prepare(	'SELECT * FROM '.$tablename.' WHERE `countrycode` = %s AND `phone` = %s AND `otp` = %s ORDER BY time DESC LIMIT 1',	$country_code,$phone,$signupotp));
	

	if($row) 

	{

		$time = strtotime($row->time);

		$current = strtotime("now");

		

		if($current - $time > 600)

		{

		$wpdb->delete($tablename, array('countrycode' => $country_code,	'phone' => $phone ), array('%d', '%s'));

		return false;

		}

		return true;

	}else 

	{

		return false;

	}

}



/* Validate otp login phone number */

add_action('wp_ajax_validate_login_phone', 'aonesms_fn_validate_login_phone');

add_action('wp_ajax_nopriv_validate_login_phone', 'aonesms_fn_validate_login_phone');

function aonesms_fn_validate_login_phone()

{

	global $wpdb, $aonesms_prefix;

	

	$country_code = aonesms_get_data($_POST,'country_code');

	$phone = aonesms_get_data($_POST,'phone');

	$phonewithcountrycode = $country_code.$phone;

	

	$row = $wpdb->get_row($wpdb->prepare('SELECT * FROM '.$wpdb->prefix.'usermeta WHERE `meta_key` = "sms_phone_with_countrycode" AND `meta_value` = %s',$phonewithcountrycode));

	

	if(!empty($row))

	{

		wp_send_json_success();

	}else{

		wp_send_json_error();

	}

	

	exit;

}

